﻿using System.Security.Claims;
using CallCenter.Api.Token;
using CallCenter.BlackLists;
using CallCenter.Calls;
using CallCenter.Ivrs;
using CallCenter.Repository.SqlSugar;
using CallCenter.Settings;
using CallCenter.Share.Requests;
using CallCenter.Tels;
using CallCenter.Users;
using IdentityModel;
using Microsoft.AspNetCore.Authorization;
using Microsoft.AspNetCore.Mvc;
using Microsoft.Extensions.Options;
using NETCore.Encrypt;
using SqlSugar;
using XF.Domain.Authentications;
using XF.Domain.Exceptions;
using XF.Domain.Options;

namespace CallCenter.Api.Controllers;

public class HomeController : BaseController
{
    private readonly IUserRepository _userRepository;
    private readonly ISugarUnitOfWork<CallCenterDbContext> _uow;
    private readonly IJwtSecurity _jwtSecurity;
    private readonly IOptionsSnapshot<IdentityConfiguration> _identityOptionsAccessor;

    public HomeController(
        IUserRepository userRepository, 
        ISugarUnitOfWork<CallCenterDbContext> uow,
        IJwtSecurity jwtSecurity,
        IOptionsSnapshot<IdentityConfiguration> identityOptionsAccessor)
    {
        _userRepository = userRepository;
        _uow = uow;
        _jwtSecurity = jwtSecurity;
        _identityOptionsAccessor = identityOptionsAccessor;
    }

    [HttpPost("login")]
    public async Task<string> Login([FromBody] LoginRequest request)
    {
        var user =
            await _userRepository.GetAsync(d => !d.IsDeleted && d.PhoneNo == request.PhoneNo, HttpContext.RequestAborted);
        if (user is null)
            throw new UserFriendlyException("未查询到该用户");

        //var token = EncryptProvider.AESEncrypt(System.Text.Json.JsonSerializer.Serialize(user), Sercurity.Key);
        var jwtOptions = _identityOptionsAccessor.Value.Jwt;
        var claims = new List<Claim>
        {
            //new(JwtClaimTypes.Id, account.Id),
            new(JwtClaimTypes.Id, user.Id),
        };
        var token = _jwtSecurity.EncodeJwtToken(claims);
        return token;
    }

    [HttpGet("createdb")]
    public Task CreateDb()
    {
        var db = _uow.Db;
        db.DbMaintenance.CreateDatabase();
        db.CodeFirst.InitTables<Call>();
        return Task.CompletedTask;
    }
}