﻿using CallCenter.BlackLists;
using CallCenter.CacheManager;
using CallCenter.Share.Dtos;
using MapsterMapper;
using Microsoft.AspNetCore.Mvc;
using XF.Domain.Cache;

namespace CallCenter.Api.Controllers;

public class ListsController : BaseController
{
    private readonly IBlacklistDomainService _blacklistDomainService;
    private readonly IBlacklistRepository _blacklistRepository;
    private readonly IMapper _mapper;

    public ListsController(
        IBlacklistDomainService blacklistDomainService,
        IBlacklistRepository blacklistRepository,
        IMapper mapper)
    {
        _blacklistDomainService = blacklistDomainService;
        _blacklistRepository = blacklistRepository;
        _mapper = mapper;
    }

    /// <summary>
    /// 添加电话号至黑名单
    /// </summary>
    /// <param name="dto"></param>
    /// <returns></returns>
    [HttpPost("blacklist")]
    public async Task AddBlacklist([FromBody] AddBlacklistDto dto)
    {
        var exists = await _blacklistRepository.AnyAsync(d => !d.IsDeleted && d.PhoneNo == dto.PhoneNo);
        if (exists) return;
        var blacklist = _mapper.Map<Blacklist>(dto);
        await _blacklistDomainService.AddAsync(blacklist, HttpContext.RequestAborted);
    }

    /// <summary>
    /// 删除黑名单数据
    /// </summary>
    /// <param name="phone"></param>
    [HttpDelete("blacklist/{phone}")]
    public void RemoveBlacklist(string phone)
    {
        _blacklistDomainService.Remove(phone);
    }

    /// <summary>
    /// 分页查询黑名单
    /// </summary>
    /// <param name="dto"></param>
    /// <returns></returns>
    [HttpGet("blacklist/paged")]
    public async Task<PagedDto<Blacklist>> QueryPaged([FromQuery] BlacklistPagedDto dto)
    {
        var (total, items) = await _blacklistRepository.QueryPagedAsync(
            d => !d.IsDeleted,
            d => d.OrderByDescending(x => x.CreationTime),
            dto.PageIndex,
            dto.PageSize,
            (!string.IsNullOrEmpty(dto.PhoneNo), d => d.PhoneNo.Contains(dto.PhoneNo!)));
        return new PagedDto<Blacklist>(total, items);
    }
}