﻿using CallCenter.Manage;
using CallCenter.Settings;
using CallCenter.Share;
using CallCenter.Share.Requests;
using Microsoft.AspNetCore.Mvc;
using XF.Domain.Exceptions;

namespace CallCenter.Api.Controllers
{
    public class SettingController : BaseController
    {
        private readonly IVoiceFileDomainService _voiceFileDomainService;
        private readonly ISystemSettingRepository _systemSettingsRepository;
        private readonly ISystemSettingGroupRepository _systemSettingGroupRepository;

        public SettingController(IVoiceFileDomainService voiceFileDomainService, ISystemSettingRepository systemSettingsRepository, ISystemSettingGroupRepository systemSettingGroupRepository)
        {
            _voiceFileDomainService = voiceFileDomainService;
            _systemSettingsRepository = systemSettingsRepository;
            _systemSettingGroupRepository = systemSettingGroupRepository;
        }

        /// <summary>
        /// 查询语音文件
        /// </summary>
        /// <returns></returns>
        [HttpGet("voicequerylist")]
        public async Task<List<string>> VoiceQueryList()
        {
            return await _voiceFileDomainService.VoiceQueryListAsync(HttpContext.RequestAborted);
        }

        /// <summary>
        /// 删除语音文件
        /// </summary>
        /// <param name="voiceFileName"></param>
        /// <returns></returns>
        [HttpPost("removevoicefile")]
        public async Task RemoveVoiceFile(string voiceFileName)
        {
            await _voiceFileDomainService.RemoveVoiceFileAsync(new RemoveVoiceFileRequest(voiceFileName), HttpContext.RequestAborted);
        }

        #region 系统参数
        /// <summary>
        /// 获取系统参数列表
        /// </summary>
        /// <returns></returns>
        [HttpGet("getsyssettings")]
        public async Task<List<SystemSettingGroup>> GetSysSettingsAsync()
        {
           //return await _systemSettingsRepository.QueryAsync(x => true);

           return await _systemSettingGroupRepository.QueryExtAsync(x => true, x => x.Includes(d => d.SystemSettings));

        }

        /// <summary>
        /// 更新配置
        /// </summary>
        /// <param name="request"></param>
        /// <returns></returns>
        /// <exception cref="UserFriendlyException"></exception>
        [HttpPost("modifysettings")]
        public async Task ModifySettings([FromBody] ModifySettingsRequest request)
        {
            for (int i = 0; i < request.list.Count; i++)
            {
                var model = await _systemSettingsRepository.GetAsync(x => x.Id == request.list[i].id,HttpContext.RequestAborted);
                if (model != null)
                {
                    model.SettingValue = request.list[i].value;
                    await _systemSettingsRepository.UpdateAsync(model, HttpContext.RequestAborted);
                }
            }
        }

        #endregion



    }
}
