﻿using CallCenter.Caches;
using CallCenter.Realtimes;
using CallCenter.Users;
using Microsoft.AspNetCore.SignalR;
using XF.Domain.Authentications;
using XF.Domain.Dependency;
using XF.Domain.Exceptions;

namespace CallCenter.Api.Realtimes;

public class CallCenterHub : Hub
{
    private readonly ISessionContext _sessionContext;
    private readonly IWorkRepository _workRepository;
    private readonly IUserCacheManager _userCacheManager;

    public CallCenterHub(ISessionContext sessionContext, IWorkRepository workRepository, IUserCacheManager userCacheManager)
    {
        _sessionContext = sessionContext;
        _workRepository = workRepository;
        _userCacheManager = userCacheManager;
    }

    /// <summary>
    /// Called when a new connection is established with the hub.
    /// </summary>
    /// <returns>A <see cref="T:System.Threading.Tasks.Task" /> that represents the asynchronous connect.</returns>
    public override async Task OnConnectedAsync()
    {
        var userId = _sessionContext.RequiredUserId;
        var work = _userCacheManager.GetWorkByUser(userId);
        if (work == null)
            throw new UserFriendlyException($"未查询到上班记录, userId: {userId}");
        work.SignalRId = Context.ConnectionId;
        await _workRepository.UpdateAsync(work, Context.ConnectionAborted);
        _userCacheManager.UpdateWorkByUser(work);
        await base.OnConnectedAsync();
    }

    /// <summary>Called when a connection with the hub is terminated.</summary>
    /// <returns>A <see cref="T:System.Threading.Tasks.Task" /> that represents the asynchronous disconnect.</returns>
    public override Task OnDisconnectedAsync(Exception? exception)
    {
        //todo 1.清理user与connectionId关联关系记录
        return base.OnDisconnectedAsync(exception);
    }
}

//public class RealtimeService : IRealtimeService, IScopeDependency
//{
//    private readonly IHubContext<CallCenterHub> _hubContext;
//    private readonly IUserCacheManager _userCacheManager;

//    public RealtimeService(IHubContext<CallCenterHub> hubContext, IUserCacheManager userCacheManager)
//    {
//        _hubContext = hubContext;
//        _userCacheManager = userCacheManager;
//    }

//    public async Task RingAsync(string userId, RingDto dto, CancellationToken cancellationToken)
//    {
//        var work = _userCacheManager.GetWorkByUser(userId);
//        if (string.IsNullOrEmpty(work.SignalRId))
//            throw new UserFriendlyException("无效signalr.connectionId");
//        await _hubContext.Clients.Client(work.SignalRId).SendAsync("Ring", dto, cancellationToken);
//    }
//}