﻿using System.Security.Authentication;
using System.Security.Claims;
using IdentityModel;
using XF.Domain.Authentications;
using XF.Domain.Dependency;

namespace CallCenter.Api.Token
{
    public class DefaultSessionContext : ISessionContext, IScopeDependency
    {
        public DefaultSessionContext(IHttpContextAccessor httpContextAccessor)
        {
            var httpContext = httpContextAccessor.HttpContext;
            if (httpContext is null)
                throw new ArgumentNullException(nameof(httpContext));
            //var a = Thread.CurrentPrincipal as ClaimsPrincipal;
            var user = httpContext.User;
            //UserId = user.Claims.FirstOrDefault(d => d.Type == "sub")?.Value ?? string.Empty;
            UserId = user.Claims.FirstOrDefault(d => d.Type == JwtClaimTypes.Id)?.Value ?? string.Empty;
            //UserName = user.Claims.FirstOrDefault(d => d.Type == "UserName")?.Value ?? string.Empty;
            //var roles = user.Claims.FirstOrDefault(d => d.Type == "role");
        }

        /// <summary>
        /// Id of current tenant or null for host
        /// </summary>
        public string? UserId { get; }

        /// <summary>
        /// Id of current user or throw Exception for guest
        /// </summary>
        /// <exception cref="AuthenticationException"></exception>
        public string RequiredUserId => UserId ?? throw new ArgumentException();

        public string? UserName { get; }

        /// <summary>
        /// Roles
        /// </summary>
        public string[] Roles { get; }

        /// <summary>
        /// Return the first value of the specific <see cref="claimType"/> claim type, otherwise null if the claim is not present.
        /// </summary>
        /// <param name="claimType"></param>
        /// <returns></returns>
        public string? FindFirstValue(string claimType)
        {
            throw new NotImplementedException();
        }
    }
}
