﻿using CallCenter.Calls;
using CallCenter.Notifications;
using CallCenter.Share.Enums;
using MediatR;

namespace CallCenter.Application.Handlers
{
    public class AlertExtToOuterNotificationHandler:INotificationHandler<AlertExtToOuterNotification>
    {
        private readonly ICallRepository _callRepository;
        private readonly ICallDetailRepository _callDetailRepository;


        public AlertExtToOuterNotificationHandler(ICallRepository callRepository, ICallDetailRepository callDetailRepository)
        {
            _callRepository = callRepository;
            _callDetailRepository = callDetailRepository;
        }

        public async Task Handle(AlertExtToOuterNotification notification, CancellationToken cancellationToken)
        {
            if (!string.IsNullOrEmpty(notification.TelNo))
            {
                var model =await _callRepository.GetAsync(x => x.ConversationId==notification.Outer.Id && x.ToNo==notification.Outer.To && x.Trunk==notification.Outer.Trunk && x.CreationTime>=DateTime.Now.AddHours(-2), cancellationToken);

                if (model!=null)
                {
                    model.CallStatus = ECallStatus.Alert;
                    await _callRepository.UpdateAsync(model, cancellationToken);
                    var detail = new CallDetail()
                    {
                        CallId = model.Id,
                        CallStatus = ECallStatus.Alert,
                        OMCallId = notification.Outer.CallId,
                        ConversationId = notification.Outer.Id,
                        EventName = notification.Attribute,
                        FromNo = notification.Outer.From,
                        ToNo = notification.Outer.To
                    };
                    await _callDetailRepository.AddAsync(detail, cancellationToken);
                }

            }

        }
    }
}
