﻿using CallCenter.Caches;
using CallCenter.Calls;
using CallCenter.Devices;
using CallCenter.Ivrs;
using CallCenter.Notifications;
using CallCenter.Settings;
using CallCenter.Share.Enums;
using MediatR;
using Microsoft.Extensions.Options;
using NewRock.Sdk;
using NewRock.Sdk.Control.Request;
using NewRock.Sdk.Transfer.Connect.Request;
using NewRock.Sdk.Transfer.Queue.Request;

namespace CallCenter.Application.Handlers
{
    public class DtmfNotificationHandler : BaseHandler, INotificationHandler<DtmfNotification>
    {
        private readonly IIvrDomainService _ivrDomainService;
        private readonly ICallRepository _callRepository;
        private readonly ICallDetailRepository _callDetailRepository;
        private readonly ICallCacheManager _callCacheManager;


        public DtmfNotificationHandler(IIvrDomainService ivrDomainService, ICallDetailRepository callDetailRepository, ICallRepository callRepository, INewRockClient newRockClient, IOptionsSnapshot<DeviceConfigs> options,ICallCacheManager callCacheManager) :base(newRockClient, options,callCacheManager, callRepository)
        {
            _ivrDomainService = ivrDomainService;
            _callDetailRepository = callDetailRepository;
            _callRepository = callRepository;
        }

        public async Task Handle(DtmfNotification notification, CancellationToken cancellationToken)
        {
            bool isvis = true;
            string menuId = string.Empty;
            string info = string.Empty;
            var model = await _callRepository.GetAsync(
                x => x.ConversationId == notification.Visitor.Id && x.FromNo == notification.Visitor.From &&
                     x.ToNo == notification.Visitor.To, cancellationToken);
            if (model == null)
            {
                model = await _callRepository.GetAsync(
                    x => x.ConversationId == notification.Outer.Id && x.FromNo == notification.Outer.From &&
                         x.ToNo == notification.Outer.To, cancellationToken);
                isvis = false;
            }
            if (model != null)
            {
                var detail = new CallDetail()
                {
                    CallId = model.Id,
                    CallStatus = ECallStatus.Dtmf,
                    EventName = notification.Attribute,
                };
                if (isvis)
                {
                    detail.OMCallId = notification.Visitor.CallId;
                    detail.ConversationId = notification.Visitor.Id;
                    detail.FromNo = notification.Visitor.From;
                    detail.ToNo = notification.Visitor.To;
                    menuId = notification.Visitor.MenuId;
                    info = notification.Visitor.Info;
                }
                else
                {
                    detail.OMCallId = notification.Outer.CallId;
                    detail.ConversationId = notification.Outer.Id;
                    detail.FromNo = notification.Outer.From;
                    detail.ToNo = notification.Outer.To;
                    menuId = notification.Outer.MenuId;
                    info = notification.Outer.Info;
                }
                detail.Remark = info;
                await _callDetailRepository.AddAsync(detail, cancellationToken);
                //调用事件处理
                var dtmfResult = await _ivrDomainService.GetDtmfAnswerAsync(menuId, info, cancellationToken);

                await base.HandlerIvr(dtmfResult, model, cancellationToken);
            }
        }
    }
}
