﻿using CallCenter.Notifications;
using CallCenter.Calls;
using CallCenter.Share.Enums;
using MediatR;

namespace CallCenter.Application.Handlers
{
    public class RingExtToOuterNotificationHandler:INotificationHandler<RingExtToOuterNotification>
    {
        private readonly ICallRepository _callRepository;
        private readonly ICallDetailRepository _callDetailRepository;

        public RingExtToOuterNotificationHandler(ICallRepository callRepository, ICallDetailRepository callDetailRepository)
        {
            _callRepository = callRepository;
            _callDetailRepository = callDetailRepository;
        }

        public async Task Handle(RingExtToOuterNotification notification, CancellationToken cancellationToken)
        {
            var model = await _callRepository.GetAsync(x =>
                x.FromNo == notification.TelNo && x.ToNo == notification.Outer.To && x.CreationTime >=DateTime.Now.AddHours(-2), cancellationToken);
            if (model==null)
            {
                model = await _callRepository.GetAsync(
                    x => x.ConversationId == notification.Outer.Id && x.ToNo == notification.Outer.To &&
                         x.Trunk == notification.Outer.Trunk, cancellationToken);
            }
                       

            if (model!=null)
            {
                model.CallStatus = ECallStatus.Ring;
                await _callRepository.UpdateAsync(model, cancellationToken);
                var detail = new CallDetail()
                {
                    CallId = model.Id,
                    CallStatus = ECallStatus.Ring,
                    EventName = notification.Attribute,
                    ConversationId = notification.Outer.Id,
                    OMCallId = notification.Outer.CallId,
                    FromNo = notification.Outer.From,
                    ToNo = notification.Outer.To,
                };
                if (string.IsNullOrEmpty(detail.FromNo))
                    detail.FromNo = notification.TelNo;

                await _callDetailRepository.AddAsync(detail, cancellationToken);
            }
        }
    }
}
