﻿using CallCenter.Caches;
using CallCenter.Calls;
using CallCenter.Realtimes;
using CallCenter.Share.Enums;
using CallCenter.Share.Notifications;
using MediatR;

namespace CallCenter.Application.Handlers
{
    public class AnsweredExtToOuterToExtNotificationHandler:INotificationHandler<AnsweredExtToOuterToExtNotification>
    {
        private readonly ICallRepository _callRepository;
        private readonly ICallDetailRepository _callDetailRepository;
        private readonly IUserCacheManager _userCacheManager;
        private readonly IRealtimeService _realtimeService;

        public AnsweredExtToOuterToExtNotificationHandler(ICallRepository callRepository, ICallDetailRepository callDetailRepository,IUserCacheManager userCacheManager, IRealtimeService realtimeService)
        {
            _callRepository = callRepository;
            _callDetailRepository = callDetailRepository;
            _userCacheManager = userCacheManager;
            _realtimeService = realtimeService;
        }

        public async Task Handle(AnsweredExtToOuterToExtNotification notification, CancellationToken cancellationToken)
        {
            var model = await _callRepository.GetAsync(x =>
                x.ConversationId == notification.Outer.Id && x.ToNo == notification.Outer.To &&
                x.Trunk == notification.Outer.Trunk && x.CreationTime >= DateTime.Now.AddHours(-2),cancellationToken);
            if (model!=null)
            {
                model.CallStatus = ECallStatus.Answered;
                await _callRepository.UpdateAsync(model, cancellationToken);
                var detail = new CallDetail()
                {
                    CallId = model.Id,
                    CallStatus = ECallStatus.Answered,
                    ConversationId = notification.Outer.Id,
                    OMCallId = notification.Outer.CallId,
                    EventName = notification.Attribute,
                    AnswerNo = notification.Outer.To,
                    FromNo = notification.Outer.From,
                    ToNo = notification.Outer.To,
                };
                await _callDetailRepository.AddAsync(detail, cancellationToken);

                //调用业务通知 通知前端
                var workModel = _userCacheManager.GetWorkByTel(notification.Outer.From);
                if (workModel != null)
                {
                    await _realtimeService.AnsweredAsync(workModel.UserId, new AnseredDto() { ConversationId = notification.Outer.Id, From = notification.Outer.From, Id = model.Id, To = notification.Outer.To, CallType= ECallType.ExtToOuter }, cancellationToken);
                }
            }
        }
    }
}
