﻿using System.Text.Json.Serialization;

namespace CallCenter.Share
{
    public record ApiResponse
    {
        public ApiResponse()
        {

        }

        public ApiResponse(int code, string message, string error)
        {
            Code = code;
            Message = message;
            Error = error;
        }

        /// <summary>
        /// 响应码
        /// </summary>
        public int Code { get; set; }

        public string? Message { get; set; }
        public string? Error { get; set; }

        /// <summary>
        /// 请求成功
        /// </summary>
        [JsonIgnore]
        public bool IsSuccess => Code == 0;

        public static ApiResponse Success() => new();
        public static ApiResponse Fail(int? code, string? message = default, string? error = default) => new(code ?? 1, message, error);
    }

    public record ApiResponse<TContent> : ApiResponse
    {
        /// <summary>
        /// 返回值
        /// </summary>
        public TContent Content { get; set; }

        public static ApiResponse Success(TContent content)
            => new ApiResponse<TContent> { Content = content };
    }
}
