﻿using CallCenter.Calls;
using XF.Domain.Cache;
using XF.Domain.Dependency;

namespace CallCenter.Caches
{
    public class CallCacheManager : ICallCacheManager, IScopeDependency
    {
        private readonly ITypedCache<IReadOnlyList<Call>> _cacheCall;
        private const string CallKey = "CallQueue";

        public CallCacheManager(ITypedCache<IReadOnlyList<Call>> cacheCall)
        {
            _cacheCall = cacheCall;
        }

        public void AddOrUpdateCallCache(List<Call> list)
        {
            _cacheCall.AddOrUpdate(CallKey, list);
        }

        public IReadOnlyList<Call> GetCallQueueList()
        {
            return _cacheCall.Get(CallKey)??new List<Call>();
        }

        public void RemoveCallCache(string id)
        {
            var list = _cacheCall.Get(CallKey)?.ToList();
            if (list != null)
            {
                var model = list.First(x => x.Id == id);
                if (model != null)
                {
                    list.Remove(model);
                    _cacheCall.AddOrUpdate(CallKey, list);
                }
            }
        }
    }
}
