﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace XF.Domain.Cache
{
    public interface ITypedCache<TValue> where TValue : class
    {
        bool Add(string key, TValue value, ExpireMode? expireMode = ExpireMode.None, TimeSpan? timeout = null);

        void Put(string key, TValue value, ExpireMode? expireMode = ExpireMode.None, TimeSpan? timeout = null);

        TValue Update(string key, Func<TValue, TValue> updateValue);

        TValue? Get(string key);

        void Expire(string key, DateTime absoluteExpiration);

        void Expire(string key, TimeSpan slidingExpiration);

        void Expire(string key, ExpireMode? expireMode = ExpireMode.None, TimeSpan? timeout = null);

        TValue AddOrUpdate(string key, TValue addValue, Func<TValue, TValue> updateValue, ExpireMode? expireMode = ExpireMode.None, TimeSpan? timeout = null);

        TValue AddOrUpdate(string key, TValue addValue, ExpireMode? expireMode = ExpireMode.None, TimeSpan? timeout = null)
            => AddOrUpdate(key, addValue, _ => addValue, expireMode, timeout);

        TValue GetOrAdd(string key, TValue value, ExpireMode? expireMode = ExpireMode.None, TimeSpan? timeout = null);

        TValue GetOrAdd(string key, Func<string, TValue> valueFactory, ExpireMode? expireMode = ExpireMode.None, TimeSpan? timeout = null);

        bool TryGetOrAdd(string key, Func<string, TValue> valueFactory, out TValue? value, ExpireMode? expireMode = ExpireMode.None, TimeSpan? timeout = null);

        bool TryUpdate(string key, Func<TValue, TValue> updateValue, out TValue? value);

        bool Remove(string key);

        bool Exists(string key);

        string CreateRegion();
    }
    
    public enum ExpireMode
    {
        /// <summary>Defines no expiration.</summary>
        None,
        /// <summary>
        /// Defines sliding expiration. The expiration timeout will be refreshed on every access.
        /// </summary>
        Sliding,
        /// <summary>
        /// Defines absolute expiration. The item will expire after the expiration timeout.
        /// </summary>
        Absolute,
    }
}
